@@ -20,7 +20,7 @@ from ocrd_models.ocrd_page import (
 )
 from ocrd import Processor
 
-from .config import get_tessdata_path, OCRD_TOOL
+from .config import OCRD_TOOL
 
 TOOL = 'ocrd-tesserocr-binarize'
 
@@ -51,9 +51,10 @@ class TesserocrBinarize(Processor):
         assert_file_grp_cardinality(self.input_file_grp, 1)
         assert_file_grp_cardinality(self.output_file_grp, 1)
 
+        sepmask = self.parameter['tiseg']
         oplevel = self.parameter['operation_level']
-        
-        with PyTessBaseAPI(path=get_tessdata_path()) as tessapi:
+
+        with PyTessBaseAPI() as tessapi:
             for n, input_file in enumerate(self.input_files):
                 file_id = make_file_id(input_file, self.output_file_grp)
                 page_id = input_file.pageId or input_file.ID
@@ -61,23 +62,40 @@ class TesserocrBinarize(Processor):
                 pcgts = page_from_file(self.workspace.download_file(input_file))
                 self.add_metadata(pcgts)
                 page = pcgts.get_Page()
-                
-                page_image, page_xywh, _ = self.workspace.image_from_page(
+                page_image, page_xywh, page_image_info = self.workspace.image_from_page(
                     page, page_id)
+                if self.parameter['dpi'] > 0:
+                    dpi = self.parameter['dpi']
+                    LOG.info("Page '%s' images will use %d DPI from parameter override", page_id, dpi)
+                elif page_image_info.resolution != 1:
+                    dpi = page_image_info.resolution
+                    if page_image_info.resolutionUnit == 'cm':
+                        dpi = round(dpi * 2.54)
+                    LOG.info("Page '%s' images will use %d DPI from image meta-data", page_id, dpi)
+                else:
+                    dpi = 0
+                    LOG.info("Page '%s' images will use DPI estimated from segmentation", page_id)
+                tessapi.SetVariable('user_defined_dpi', str(dpi))
                 LOG.info("Binarizing on '%s' level in page '%s'", oplevel, page_id)
 
                 if oplevel == 'page':
                     tessapi.SetPageSegMode(PSM.AUTO_ONLY)
                     tessapi.SetImage(page_image)
+                    if sepmask:
+                        # will trigger FindLines() → SegmentPage() → AutoPageSeg()
+                        # → SetupPageSegAndDetectOrientation() → FindAndRemoveLines() + FindImages()
+                        tessapi.AnalyseLayout()
                     page_image_bin = tessapi.GetThresholdedImage()
                     if page_image_bin:
                         # update METS (add the image file):
                         file_path = self.workspace.save_image_file(page_image_bin,
                                                                    file_id + '.IMG-BIN',
-                                                                   page_id=page_id,
+                                                                   page_id=input_file.pageId,
                                                                    file_grp=self.output_file_grp)
                         # update PAGE (reference the image file):
                         features = page_xywh['features'] + ",binarized"
+                        if sepmask:
+                            features += ",clipped"
                         page.add_AlternativeImage(AlternativeImageType(
                             filename=file_path, comments=features))
                     else:
