@@ -27,7 +27,7 @@ from ocrd_models.ocrd_page import (
 )
 from ocrd import Processor
 
-from .config import get_tessdata_path, OCRD_TOOL
+from .config import OCRD_TOOL
 
 TOOL = 'ocrd-tesserocr-deskew'
 
@@ -62,7 +62,6 @@ class TesserocrDeskew(Processor):
         oplevel = self.parameter['operation_level']
         
         with PyTessBaseAPI(
-                path=get_tessdata_path(),
                 lang="osd", # osd required for legacy init!
                 oem=OEM.TESSERACT_LSTM_COMBINED, # legacy required for OSD!
                 psm=PSM.AUTO_OSD
@@ -95,8 +94,7 @@ class TesserocrDeskew(Processor):
                 else:
                     dpi = 0
                     LOG.info("Page '%s' images will use DPI estimated from segmentation", page_id)
-                if dpi:
-                    tessapi.SetVariable('user_defined_dpi', str(dpi))
+                tessapi.SetVariable('user_defined_dpi', str(dpi))
                 
                 LOG.info("Deskewing on '%s' level in page '%s'", oplevel, page_id)
                 
