@@ -3,10 +3,12 @@ from os.path import join
 from sys import exit, stdout
 from os import scandir
 import math
+import itertools
 from PIL import Image, ImageStat
 import numpy as np
-from shapely.geometry import Polygon, asPolygon
-from shapely.ops import unary_union
+from scipy.sparse.csgraph import minimum_spanning_tree
+from shapely.geometry import Polygon, LineString
+from shapely.ops import unary_union, nearest_points
 from shutil import copyfileobj
 from pathlib import Path
 
@@ -16,7 +18,8 @@ from tesserocr import (
     WritingDirection,
     TextlineOrder,
     tesseract_version,
-    PyTessBaseAPI, get_languages as get_languages_)
+    PyTessBaseAPI,
+    get_languages)
 
 from ocrd_utils import (
     getLogger,
@@ -62,7 +65,7 @@ from ocrd_models.ocrd_page_generateds import (
 from ocrd_modelfactory import page_from_file
 from ocrd import Processor
 
-from .config import get_tessdata_path, OCRD_TOOL
+from .config import OCRD_TOOL
 
 TOOL = 'ocrd-tesserocr-recognize'
 
@@ -70,12 +73,6 @@ CHOICE_THRESHOLD_NUM = 10 # maximum number of choices to query and annotate
 CHOICE_THRESHOLD_CONF = 1 # maximum score drop from best choice to query and annotate
 # (ChoiceIterator usually rounds to 0.0 for non-best, so this better be maximum)
 
-def get_languages(*args, **kwargs):
-    """
-    Wraps tesserocr.get_languages() with a fixed path parameter.
-    """
-    return get_languages_(*args, path=get_tessdata_path(), **kwargs)
-
 # monkey-patch the tesserocr base class so have at least some state
 class TessBaseAPI(PyTessBaseAPI):
     parameters = {}
@@ -135,32 +132,15 @@ class TesserocrRecognize(Processor):
         kwargs['ocrd_tool'] = OCRD_TOOL['tools'][TOOL]
         kwargs['version'] = OCRD_TOOL['version'] + ' (' + tesseract_version().split('\n')[0] + ')'
 
-        if kwargs.get('show_resource', False):
-            initLogging()
-            resdir = get_tessdata_path()
-            fpath = kwargs['show_resource']
-            if not fpath.endswith('.traineddata'):
-                fpath += '.traineddata'
-            fpath = Path(resdir, fpath)
-            try:
-                with open(fpath, 'rb') as f:
-                    copyfileobj(f, stdout.buffer)
-                exit(0)
-            except FileNotFoundError as e:
-                getLogger('processor.TesserocrRecognize').error("Resource %s does not exist", fpath)
-                exit(1)
-
-        if kwargs.get('list_resources', False):
-            initLogging()
-            resdir = get_tessdata_path()
-            for res in scandir(resdir):
-                print(join(resdir, res.name))
-            exit(0)
         super(TesserocrRecognize, self).__init__(*args, **kwargs)
         
         if hasattr(self, 'workspace'):
             self.logger = getLogger('processor.TesserocrRecognize')
 
+    @property
+    def moduledir(self):
+        return get_languages()[0]
+
     def process(self):
         """Perform layout segmentation and/or text recognition with Tesseract.
         
@@ -370,8 +350,7 @@ class TesserocrRecognize(Processor):
             for variable in tesseract_params:
                 tessapi.SetVariable(variable, tesseract_params[variable])
             # Initialize Tesseract (loading model)
-            tessapi.InitFull(path=get_tessdata_path(),
-                             lang=model,
+            tessapi.InitFull(lang=model,
                              oem=getattr(OEM, self.parameter['oem']))
             # Iterate input files
             for (n, input_file) in enumerate(self.input_files):
@@ -400,8 +379,7 @@ class TesserocrRecognize(Processor):
                     dpi = 0
                     self.logger.info("Page '%s' images will use DPI estimated from segmentation",
                                      page_id)
-                if dpi:
-                    tessapi.SetVariable('user_defined_dpi', str(dpi))
+                tessapi.SetVariable('user_defined_dpi', str(dpi))
                 
                 self.logger.info("Processing page '%s'", page_id)
                 # FIXME: We should somehow _mask_ existing regions in order to annotate incrementally (not redundantly).
@@ -459,7 +437,7 @@ class TesserocrRecognize(Processor):
                     page_image_bin = tessapi.GetThresholdedImage()
                     file_path = self.workspace.save_image_file(
                         page_image_bin, file_id + '.IMG-BIN',
-                        page_id=page_id,
+                        page_id=input_file.pageId,
                         file_grp=self.output_file_grp)
                     # update PAGE (reference the image file):
                     page.add_AlternativeImage(AlternativeImageType(
@@ -1469,16 +1447,41 @@ def join_segments(segments):
     return join_polygons([polygon_from_points(segment.get_Coords().points)
                           for segment in segments])
 
-def join_polygons(polygons, extend=2):
-    # FIXME: construct concave hull / alpha shape
-    jointp = unary_union([make_valid(Polygon(polygon)).buffer(extend)
-                          for polygon in polygons]).convex_hull
+def join_polygons(polygons, scale=20):
+    """construct concave hull (alpha shape) from input polygons by connecting their pairwise nearest points"""
+    # ensure input polygons are simply typed
+    polygons = list(itertools.chain.from_iterable([
+        poly.geoms if poly.type in ['MultiPolygon', 'GeometryCollection']
+        else [poly]
+        for poly in polygons]))
+    npoly = len(polygons)
+    if npoly == 1:
+        return polygons[0]
+    # find min-dist path through all polygons (travelling salesman)
+    pairs = itertools.combinations(range(npoly), 2)
+    dists = np.zeros((npoly, npoly), dtype=float)
+    for i, j in pairs:
+        dist = polygons[i].distance(polygons[j])
+        if dist == 0:
+            dist = 1e-5 # if pair merely touches, we still need to get an edge
+        dists[i, j] = dist
+        dists[j, i] = dist
+    dists = minimum_spanning_tree(dists, overwrite=True)
+    # add bridge polygons (where necessary)
+    for prevp, nextp in zip(*dists.nonzero()):
+        prevp = polygons[prevp]
+        nextp = polygons[nextp]
+        nearest = nearest_points(prevp, nextp)
+        bridgep = LineString(nearest).buffer(max(1, scale/5), resolution=1)
+        polygons.append(bridgep)
+    jointp = unary_union(polygons)
+    assert jointp.type == 'Polygon', jointp.wkt
     if jointp.minimum_clearance < 1.0:
         # follow-up calculations will necessarily be integer;
         # so anticipate rounding here and then ensure validity
-        jointp = asPolygon(np.round(jointp.exterior.coords))
+        jointp = Polygon(np.round(jointp.exterior.coords))
         jointp = make_valid(jointp)
-    return jointp.exterior.coords[:-1]
+    return jointp
 
 def pad_image(image, padding):
     # TODO: input padding can create extra edges if not binarized; at least try to smooth
@@ -1522,7 +1525,14 @@ def polygon_for_parent(polygon, parent):
     if childp.within(parentp):
         return childp.exterior.coords[:-1]
     # clip to parent
-    interp = childp.intersection(parentp)
+    interp = make_intersection(childp, parentp)
+    if not interp:
+        return None
+    return interp.exterior.coords[:-1] # keep open
+
+def make_intersection(poly1, poly2):
+    interp = poly1.intersection(poly2)
+    # post-process
     if interp.is_empty or interp.area == 0.0:
         # this happens if Tesseract "finds" something
         # outside of the valid Border of a deskewed/cropped page
@@ -1533,27 +1543,27 @@ def polygon_for_parent(polygon, parent):
         interp = unary_union([geom for geom in interp.geoms if geom.area > 0])
     if interp.type == 'MultiPolygon':
         # homogeneous result: construct convex hull to connect
-        # FIXME: construct concave hull / alpha shape
-        interp = interp.convex_hull
+        interp = join_polygons(interp.geoms)
     if interp.minimum_clearance < 1.0:
         # follow-up calculations will necessarily be integer;
         # so anticipate rounding here and then ensure validity
-        interp = asPolygon(np.round(interp.exterior.coords))
+        interp = Polygon(np.round(interp.exterior.coords))
         interp = make_valid(interp)
-    return interp.exterior.coords[:-1] # keep open
+    return interp
 
 def make_valid(polygon):
-    for split in range(1, len(polygon.exterior.coords)-1):
+    points = list(polygon.exterior.coords)
+    for split in range(1, len(points)):
         if polygon.is_valid or polygon.simplify(polygon.area).is_valid:
             break
         # simplification may not be possible (at all) due to ordering
         # in that case, try another starting point
-        polygon = Polygon(polygon.exterior.coords[-split:]+polygon.exterior.coords[:-split])
-    for tolerance in range(1, int(polygon.area)):
+        polygon = Polygon(points[-split:]+points[:-split])
+    for tolerance in range(int(polygon.area)):
         if polygon.is_valid:
             break
         # simplification may require a larger tolerance
-        polygon = polygon.simplify(tolerance)
+        polygon = polygon.simplify(tolerance + 1)
     return polygon
 
 def iterate_level(it, ril, parent=None):
